﻿// The map copyright label by default.
var ATTRIBUTION = "&copy; <a href='https://www.openstreetmap.org/copyright' target='_blank'>OpenStreetMap</a> contributors";
// The number of decimal digits in coordinates.
var COORD_DIGITS = 6;
// The value that means normal location status.
var NORMAL_VAL = 0;
// The value that means error location status.
var ERROR_VAL = 1;
// THe unreliable channel status.
var UNREL_STAT = 5;
// The max. width of map popups.
var POPUP_MAX_WIDTH = 500;

// The view hub object
var viewHub = scada.viewHubLocator.getViewHub();
// Notifier control
var notifier = null;
// Input channel filter for current data requests
var cnlFilter = null;
// Dictionary of the current data
var cnlDataMap = null;
// Markers of the map
var mapMarkers = [];
// Marker icons
var grayMarkerIcon = null;
var greenMarkerIcon = null;
var redMarkerIcon = null;

// The variables below must be defined in Map.aspx
// View ID
var viewID = viewID || 0;
// View in JSON format
var viewJson = viewJson || "{}";
// Current data refresh rate
var dataRefrRate = dataRefrRate || 1000;
// Localized phrases
var phrases = phrases || {};

// Update layout of the top level div elements
function updateLayout() {
    var divNotif = $("#divNotif");
    var divMap = $("#divMap");
    var notifHeight = divNotif.css("display") === "block" ? divNotif.outerHeight() : 0;
    var windowWidth = $(window).width();

    $("body").css("padding-top", notifHeight);
    divNotif.outerWidth(windowWidth);
    divMap.outerHeight($(window).height() - notifHeight);
}

// Initialize the marker icons
function initMarkerIcons() {
    grayMarkerIcon = new L.Icon({
        iconUrl: 'images/marker-icon-2x-gray.png',
        shadowUrl: 'images/marker-shadow.png',
        iconSize: [25, 41],
        iconAnchor: [12, 41],
        popupAnchor: [1, -34],
        shadowSize: [41, 41]
    });

    greenMarkerIcon = new L.Icon({
        iconUrl: 'images/marker-icon-2x-green.png',
        shadowUrl: 'images/marker-shadow.png',
        iconSize: [25, 41],
        iconAnchor: [12, 41],
        popupAnchor: [1, -34],
        shadowSize: [41, 41]
    });

    redMarkerIcon = new L.Icon({
        iconUrl: 'images/marker-icon-2x-red.png',
        shadowUrl: 'images/marker-shadow.png',
        iconSize: [25, 41],
        iconAnchor: [12, 41],
        popupAnchor: [1, -34],
        shadowSize: [41, 41]
    });
}

// Build and display map
function displayMap() {
    var mapView = $.parseJSON(viewJson);
    var tiling = mapView.Tiling;
    var initialView = mapView.InitialView;

    // show title
    setTitle(mapView.Title);

    // prepare map
    var mapObj = L.map("divMap").setView([initialView.Lat, initialView.Lon], initialView.Zoom);

    if (typeof tiling.Options.attribution === "undefined") {
        tiling.Options.attribution = ATTRIBUTION;
    }

    L.tileLayer(tiling.UrlTemplate, tiling.Options).addTo(mapObj);

    // add markers
    mapMarkers = [];
    for (var location of mapView.Locations) {
        var marker = createMarker(location);
        marker.addTo(mapObj);
        mapMarkers.push(marker);
    }

    // show coordinates on a map click
    var mapPopup = L.popup();
    mapObj.on("click", function (e) {
        mapPopup
            .setLatLng(e.latlng)
            .setContent(phrases.ClickPosition + e.latlng.lat.toFixed(COORD_DIGITS) + ", " +
                e.latlng.lng.toFixed(COORD_DIGITS))
            .openOn(mapObj);
    });

    // display up-to-date data when a popup opens
    mapObj.on("popupopen", function (e) {
        displayDataItems();
        e.popup.update();
    });

    // show chart on a data item click
    // bind to body because marker elements don't exist here
    $("body").on("click", ".marker-popup .popup-data-table td.data-item-val", function () {
        var cnlNum = parseInt($(this).attr("data-cnl"));
        showChart(cnlNum);
    });
}

// Set window title
function setTitle(viewTitle) {
    if (viewTitle && viewHub) {
        document.title = viewTitle + " - " + viewHub.getEnv().productName;
        viewHub.notify(scada.EventTypes.VIEW_TITLE_CHANGED, window, document.title);
    }
}

// Show chart of the specified input channel
function showChart(cnlNum) {
    if (viewHub && viewHub.dialogs) {
        var date = viewHub.curViewDateMs ? new Date(viewHub.curViewDateMs) : new Date();
        viewHub.dialogs.showChart(cnlNum, viewID, date);
    } else {
        console.warn("Dialogs object is undefined");
    }
}

// Create new marker according to the location settings
function createMarker(location) {
    var marker = L.marker([location.Lat, location.Lon]);
    marker.statusCnlNum = location.StatusCnlNum;

    if (marker.statusCnlNum > 0) {
        marker.setIcon(grayMarkerIcon);
    }

    // create a popup bound to the marker
    var divMarkerPopup = $("<div class='marker-popup'></div>");
    $("<div class='popup-title'></div>").text(location.Name).appendTo(divMarkerPopup);
    $("<div class='popup-descr'></div>").text(location.Descr).appendTo(divMarkerPopup);

    // data items
    if (location.DataItems.length > 0) {
        var tblDataItems = $("<table class='popup-data-table'></table>").appendTo(divMarkerPopup);

        for (var dataItem of location.DataItems) {
            var row = $("<tr></tr>").appendTo(tblDataItems);
            $("<td class='data-item-cap'></td>").text(dataItem.Caption + ":").appendTo(row);
            $("<td class='data-item-val'></td>").attr("data-cnl", dataItem.CnlNum).appendTo(row);
        }
    }

    // link
    if (location.Link && location.Link.ViewID > 0) {
        var url = viewHub ? viewHub.getFullViewUrl(location.Link.ViewID) : "#";
        $("<div class='popup-footer'><a href='" + url + "' target='_top'>" +
            phrases.DetailsLink + "</a><div>").appendTo(divMarkerPopup);
    }

    marker.bindPopup(divMarkerPopup[0], { maxWidth: POPUP_MAX_WIDTH });
    return marker;
}

// Start cyclic update of data items
function startUpdatingData() {
    updateData(function (success) {
        if (!success) {
            notifier.addNotification(phrases.UpdateCurDataError, scada.NotifTypes.ERROR, notifier.DEF_NOTIF_LIFETIME);
        }

        setTimeout(startUpdatingData, dataRefrRate);
    });
}

// Request and display current data.
// callback is a function (success)
function updateData(callback) {
    scada.clientAPI.getCurCnlDataExt(cnlFilter, function (success, cnlDataExtArr) {
        if (success) {
            cnlDataMap = scada.clientAPI.createCnlDataExtMap(cnlDataExtArr);
            displayLocationStatus();
            displayDataItems();
            callback(true);
        } else {
            callback(false);
        }
    });
}

// Display status of locations by changing marker icon
function displayLocationStatus() {
    for (var marker of mapMarkers) {
        if (marker.statusCnlNum > 0) {
            var cnlData = cnlDataMap.get(marker.statusCnlNum);
            var markerIcon = grayMarkerIcon;

            if (cnlData && cnlData.Stat > 0 && cnlData.Stat !== UNREL_STAT) {
                if (cnlData.Val === NORMAL_VAL) {
                    markerIcon = greenMarkerIcon;
                } else if (cnlData.Val === ERROR_VAL) {
                    markerIcon = redMarkerIcon;
                }
            }

            if (marker.icon !== markerIcon) {
                marker.setIcon(markerIcon);
            }
        }
    }
}

// Display values of data items of the visible popups
function displayDataItems() {
    if (cnlDataMap) {
        var dataCells = $(".marker-popup .popup-data-table td.data-item-val");

        dataCells.each(function () {
            var cell = $(this);
            var cnlNum = parseInt(cell.attr("data-cnl"));

            if (cnlNum) {
                var cnlData = cnlDataMap.get(cnlNum);

                if (cnlData) {
                    cell.text(cnlData.TextWithUnit);
                    cell.css("color", cnlData.Color);
                } else {
                    cell.text("");
                    cell.css("color", "");
                }
            }
        });
    }
}

$(document).ready(function () {
    // setup client API
    scada.clientAPI.rootPath = "../../";
    scada.clientAPI.ajaxQueue = scada.ajaxQueueLocator.getAjaxQueue();

    // setup user interface
    updateLayout();
    notifier = new scada.Notifier("#divNotif");
    notifier.startClearingNotifications();

    $(window).on("resize " + scada.EventTypes.UPDATE_LAYOUT, function() {
        updateLayout();
    });

    // display map
    initMarkerIcons();
    displayMap();

    // start updating data
    cnlFilter = new scada.CnlFilter();
    cnlFilter.viewID = viewID;
    startUpdatingData();
});